package vt.CS5934.SwitchRoom.services;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import vt.CS5934.SwitchRoom.models.StateCityLookupModel;
import vt.CS5934.SwitchRoom.models.UserOfferModel;
import vt.CS5934.SwitchRoom.repositories.StateCityLookupRepository;
import vt.CS5934.SwitchRoom.repositories.UserOfferRepository;
import vt.CS5934.SwitchRoom.utility.StateCodeMap;

@Service
public class OfferPageService {
    private final Logger logger = LoggerFactory.getLogger(OfferPageService.class);

    @Autowired
    UserOfferRepository userOfferRepository;
    @Autowired
    StateCityLookupRepository stateCityLookupRepository;

    public UserOfferModel getUserOfferInfo(Long userId) {
        UserOfferModel userOfferModel = userOfferRepository.findByUserId(userId);
        if(userOfferModel == null) {
            userOfferModel = new UserOfferModel();
            userOfferModel.setUserId(userId);
        }
        return userOfferModel;
    }


    public UserOfferModel saveNewUserOffer(UserOfferModel offerModel) {
        fillStateCodeAndStateCityCode(offerModel);
        return userOfferRepository.save(offerModel);
    }

    public void deleteUserOffer(Long userId) {
        userOfferRepository.deleteByUserId(userId);
    }


    /**
     * This function will take a UserOfferModel. it will fill the StateCode with it first,
     * then check and update the StateCityCode. If DB do not have the StateCityCode, then build one and fill it in.
     * @param offerModel the new or updated offerModel
     */
    private void fillStateCodeAndStateCityCode(UserOfferModel offerModel){

        // Fill state code in the record
        offerModel.setStateCode(StateCodeMap.StringToCodeMap.get(offerModel.getState()));

        // Check if the DB have the state city code or not, if not build one
        StateCityLookupModel stateCityLookupModel = stateCityLookupRepository
                .findByCityNameAndStateCode(offerModel.getSpaceLocateCity(), offerModel.getStateCode());

        if(stateCityLookupModel == null){
            // not in DB yet
            stateCityLookupModel = stateCityLookupRepository.save(
                    new StateCityLookupModel(null, offerModel.getStateCode(),
                            offerModel.getSpaceLocateCity()));
        }
        offerModel.setStateCityCode(stateCityLookupModel.getStateCityCode());
    }
}
/*
Change logs:
Date        |       Author          |   Description
2022-10-21  |    Fangzheng Zhang    |    create class and init
2022-10-30  |   Fangzheng Zhang     |   implement state city code feature

 */